;;;;;;;;;;;;;;;;;;;;;;hexview.asm
;From pgm@cgmsun.physics.utoledo.eduSun Dec 17 23:28:59 1995
;Date: Wed, 13 Dec 95 16:24:16 EST
;From: Philip Montgomery <pgm@cgmsun.physics.utoledo.edu>
;Reply to: list-zshell@kuai.se
;To: list-zshell@kuai.se
;Subject: Re: LZ: hex editor for zshell
;
;Okay, here's some example code for a hex viewer that always you to
;scroll up and down through memory with the up, down, and two other keys
;I forget ...  Oh yeah, + and - jump up and down by 256 bytes at a time.
;Anyway, this was my first Zshell program, put here for educational
;purposes.

;Press [ALPHA] to see the text in the right most column.  Sometimes it
;messes up the screen though.  Use * and / to jump by 0x1000.
;The keys are situated like you were using a text editor.
;So up scrolls up but decrements the addresses.

;;;;;;;;;;;;;;;;;;;
;;;History:
;;;version 0.0: initial release by Philip Montgomery
;;;        0.1: Added star and slash big jumps by $1000 and added "alpha"
;;;             mode.  ZShell 4.0 compatable. - Chris Busch
;;; Version 0.1 recompiled for Ash By Dines Justesen       
; Translated to OShell-82 by Thomas J. Hruska

#include "offti-82.h"
; Copyright 1995, Philip Montgomery
; pgm@cgmsun.physics.utoledo.edu

; Definitions of keys

K_UP    = $4
K_DOWN  = $1
K_PLUS  = $A
K_MINUS = $B
K_STAR  = $C
K_SLASH = $D
K_ALPHA = $30
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;text memory
CmdY      =TEXT_MEM   ;$80DF   ; y position
CmdX      =CmdY+1     ;$80E0   ; x position
HexPtr    =CmdX+1     ;$80E1
CurPtr    =HexPtr+2   ;$80E3   ; ptr to first dump
curmode   =CurPtr+2   ;$80E5   ;current mode bits
savehl    =curmode+1  ;$80E6   ;save hl for char dump
lastvar   =savehl+2   ;$80E8


;;bit modes: curmode has the following modes in it.
alphamode =0  ;print the ascii in the right column, set=yes

.org 0
#IFDEF OSHELL82
.DB "Hex Viewer",0
#ENDIF

	ld hl,0
	ld (CmdY),hl
	ld (CmdX),hl
	ld (HexPtr),hl
	ld (CurPtr),hl
	ld (curmode),hl
	ld (savehl),hl
	ld (lastvar),hl
main:
	ROM_CALL(CLEARLCD)

	ld   a,0
	ld (CURSOR_X),a ;hl
	ld (CURSOR_Y),a ;hl

	ld hl, (HexPtr)
	CALL_(DrawScreen)

NotDone:
	call GET_KEY
	cp $37
	ret z
	cp K_UP        ; down
	jr z, DrawDown
	cp K_DOWN      ; up
	jr z, DrawUp
	cp K_PLUS
	jr z, JumpDown
	cp K_MINUS
	jr z, JumpUp
	cp K_STAR
	jr z, bigjumpdn
	cp K_SLASH
	jr z, bigjumpup
	cp K_ALPHA
	CALL_Z(setalpha)
	jr NotDone


DrawDown:
	ld hl, -8
DoAdvance:
	ld de, (CurPtr)          ; (CurPtr) -> de
	add hl, de               ; hl = hl + de
	ld (CurPtr), hl          ; hl -> (CurPtr)
	ld a,(curmode)           ; (curmode) -> a
	bit alphamode,a          ; Test bit 0 of a
	jr  z,skipclr            ; If bit 0 is reset (0) then jump to skipclr
	ROM_CALL(CLEARLCD)       ; Clear the LCD
skipclr:
	CALL_(DrawScreen)        ; Refresh the screen
	jr NotDone               ; Jump to NotDone

DrawUp:
	ld hl, 8
	jr DoAdvance

JumpDown:
	ld hl, 0100h
	jr DoAdvance

JumpUp:
	ld hl, - 0100h
	jr DoAdvance

bigjumpdn:
	ld hl, 1000h
	jr DoAdvance

bigjumpup:
	ld hl, - 1000h
	jr DoAdvance


;;;;;;;;drawscreen function
DrawScreen:
	push hl                  ; Temporarily store hl on the stack
	sub a                    ; a = a - a  (EQU ld a,0)
	ld (CURSOR_X), a         ; Moves to (0,0) on the graph screen
	ld (CURSOR_Y), a

	ld hl, (CurPtr)          ; Supposedly load $0000 -> hl
	ld b, 8                  ; 8 -> b
NextRow:
	CALL_(DisplayLine)       ; Calls DisplayLine
	ld a, (CURSOR_Y)         ; (CURSOR_Y) -> a
	add a, 7                 ; a = a + 7
	ld (CURSOR_Y), a         ; a -> (CURSOR_Y)
	sub a                    ; a = a - a (EQU ld a,0)
	ld (CURSOR_X), a         ; a -> (CURSOR_X)
	dec b                    ; b = b - 1
	jr nz, NextRow           ; if b<>0 then jump to NextRow

	pop hl                   ; Retrieve hl from the stack
	ret                      ; Return to caller
;;end drawscreen

;;;;;function displayline
DisplayLine:                  ; hl -> start of dump
	push de                  ; Temporarily store de on the stack
	push bc                  ; Temporarily store bc on the stack

	ld a, h                  ; h -> a
	CALL_(PrintHex)          ; Calls PrintHex
	ld a, l                  ; l -> a
	CALL_(PrintHex)          ; Print address being decoded

	ld b, 3                  ; 3 -> b
NextBlank:                    ; Puts 3 spaces on the screen
	ld a, ' '                ; ' ' -> a  (IS THIS LEGAL?)
	ROM_CALL(M_CHARPUT)      ; Draw the space
	dec b                    ; b = b - 1
	jr nz, NextBlank         ; if b<>0 then jump to NextBlank
	ld (savehl),hl           ; hl -> (savehl)   (saves hl for later use)
	ld b, 8                  ; 8 -> b
NextChar:                     ; Puts 8 bytes on the screen
	ld a, (hl)               ; (hl) -> a
	inc hl                   ; hl = hl + 1
	CALL_(PrintHex)          ; Calls PrintHex
	dec b                    ; b = b - 1
	jr nz, NextChar          ; if b<>0 then jump to NextChar
	ld a,(curmode)           ; (curmode) -> a
	bit alphamode,a          ; Test bit 0 of a
	jr z,noalphamode         ; If bit 0 is reset (0) then goto noalphamode
	ld hl,(savehl)           ; (savehl) -> hl   (restores hl)
	ld b, 8                  ; 8 -> b
NextChar0:                    ; Puts 8 bytes on the screen
	ld a, (hl)               ; (hl) -> a
	inc hl                   ; hl = hl + 1
	ROM_CALL(M_CHARPUT)      ; Display the character
	dec b                    ; b = b - 1
	jr nz, NextChar0         ; if b<>0 then jump to NextChar0
noalphamode:
	pop bc                   ; Retrieve bc from the stack
	pop de                   ; Retrieve de from the stack
	ret                      ; Return to caller
;;;end display line


;;;;;;;;;;;printhex function
PrintHex:                     ; de destroyed
	push bc                  ; Temporarily store bc on the stack
	push de                  ; Temporarily store de on the stack

	ld b, a                  ; a -> b
	and $F0                  ; Mask out upper four bits
	rrca                     ; Shift the upper bits to the lower 4 bit
	rrca                     ;  positions
	rrca
	rrca

	add a, 30h               ; a = a + $30 (location of numbers)
	cp $3a                   ; If lower bits have hex letters carry isn't set
	jr c, hexnumber0         ; If carry set then jump to hexnumber0
	add a, $7                ; a = a + $7  (location of letters)
hexnumber0:
	ROM_CALL(M_CHARPUT)      ; Display the character

	ld a, b                  ; b -> a
	and $0f                  ; Mask out the lower four bits

	add a, 30h               ; a = a +$30 (location of numbers)
	cp $3a                   ; If lower bits have hex letters carry isn't set
	jr c, hexnumber1         ; If carry set then jump to hexnumber1
	add a, $7                ; a = a + $7  (location of letters)
hexnumber1:
	ROM_CALL(M_CHARPUT)      ; Display the character
	pop de                   ; Retrieve de from the stack
	pop bc                   ; Retrieve bc from the stack
	ret                      ; Return to caller
;;;;;;;;;end printhex function

;;;void setalpha()
;;destroys a
setalpha:
   push af                    ; Temporarily store af on the stack
   ROM_CALL(CLEARLCD)         ; Clear the LCD
   ld  a,(curmode)            ; (curmode) -> a
   bit alphamode,a            ; Test bit 0 of a
   set alphamode,a            ; Set (1) bit 0 of a
   jr  z,noclralpha           ; If bit 0 is reset (0) then jump to noclralpha
   res alphamode,a            ; Reset (0) bit 0 of a
noclralpha:
   ld  (curmode),a            ; a -> (curmode)
   CALL_(DrawScreen)          ; Refresh the screen
   pop  af                    ; Retrieve af from the stack
   ret                        ; Return to caller
;;end setalpha

.END
