;The Official TI-82.H include file for
;OShell-82 v2.5 and ASH v2.0 programming
;Designed to make OShell-82 and ASH compatible
;(C) 1997 Shining Light Productions,
;original text modified by Thomas J. Hruska

#DEFINE OSHELL82     ; This must be defined if compiling for OShell-82

#IFDEF OSHELL82
;OShell-82 v2.5 TASM include file
;(C) 1997 Alphasoft, written by Jason Todd

; ROM_CALL2 destroys the flags only!!!
; Use ROM_CALL2 sparingly as it uses 20 bytes instead of 5
#DEFINE ROM_CALL2(addr)  push bc \ push af \ ld a,($29D0) \ cp $6D \ pop bc \ ld a,b \ pop bc \ call z,addr+$1A \ call nz,addr

#DEFINE ROM_CALL(INDEX) CALL $8D28 \ .DB INDEX
#DEFINE CALL_Z(ADDR)    CALL $8D2B \ .DW ADDR
#DEFINE CALL_(ADDR)     CALL $8D2E \ .DW ADDR
#DEFINE CALL_NZ(ADDR)   CALL $8D31 \ .DW ADDR
#DEFINE CALL_C(ADDR)    CALL $8D37 \ .DW ADDR
#DEFINE CALL_NC(ADDR)   CALL $8D3D \ .DW ADDR
#DEFINE JUMP_Z(ADDR)    CALL $8D43 \ .DW ADDR
#DEFINE JUMP_(ADDR)     CALL $8D46 \ .DW ADDR
#DEFINE JUMP_NZ(ADDR)   CALL $8D49 \ .DW ADDR
#DEFINE JUMP_C(ADDR)    CALL $8D4F \ .DW ADDR
#DEFINE JUMP_NC(ADDR)   CALL $8D55 \ .DW ADDR

;OShell-82 v2.0 Public data areas/variables
PROGRAM_ADDR    =$8D5B  ; RAM location of currently executing program
OS_VER          =$8D5D  ; OShell version number (top 4 bits = major version)
OS_BITS         =$8D5E  ; OShell status byte (currently only used internally)

;The following lines have been remarked out because they aren't needed
;These names are reserved for the ROM_CALL() macro
TX_CHARPUT       =$00   ; Normal text display translated CHAR of A
D_LT_STR         =$01   ; Normal text display length-indexed string at HL
M_CHARPUT        =$02   ; Small text display CHAR of A
D_ZM_STR         =$03   ; Small text display ASCIIZ string at HL
SCROLL_UP        =$06   ; Scroll the screen up one line
TR_CHARPUT       =$07   ; Normal text display untranslated CHAR of A
CLEARLCD         =$08   ; Clear the LCD hardware only
D_HL_DECI        =$09   ; Normal text display right justified VALUE of HL
CLEARTEXT        =$0A   ; Set all the text memory to 00
CLEARTEXT_F      =$0A   ; Set all the text memory to 00
D_ZT_STR         =$0B   ; Normal text display ASCIIZ string at HL
BUSY_OFF         =$0C   ; Turn off "busy" indicator
BUSY_ON          =$0D   ; Turn on "busy" indicator
;_GETKEY          =$0E   ; Wait for and return getkey code (See KEYCODES.INC)
;_GRBUFCPY_V      =$0F   ; Copies graph screen to LCD

;VAT_START has a conflicting address with VAT_START for ASH
;TI-82 RAM data areas/variables - I will expand on this later
VAT_START       =$FE6F  ; Beginning of VAT (works backwards)

#ELSE
;ASH v2.0 TASM include file

#DEFINE CALL_(addr) CALL $8D76 \ .DW addr
#DEFINE JUMP_(addr) CALL $8D86 \ .DW addr
#DEFINE CALL_Z(addr) CALL $8D74 \ .DW addr
#DEFINE JUMP_Z(addr) CALL $8D84 \ .DW addr
#DEFINE CALL_NZ(addr) CALL $8D78 \ .DW addr
#DEFINE JUMP_NZ(addr) CALL $8D88 \ .DW addr
#DEFINE CALL_C(addr) CALL $8D7C \ .DW addr
#DEFINE JUMP_C(addr) CALL $8D8C \ .DW addr
#DEFINE CALL_NC(addr) CALL $8D80 \ .DW addr
#DEFINE JUMP_NC(addr) CALL $8D90 \ .DW addr
#DEFINE ROM_CALL(addr) CALL $8D96 \ .DW addr

#DEFINE ROM_CALL2(addr) CALL $8D96 \ .DW addr

PROGRAM_ADDR = $8D94

;RAM locations
VAT_START      =$FE6E    ; start of VAT (conflicts with VAT_START of OSHELL)

TX_CHARPUT      =$39EC-$1A  ; xlated display of %A on screen, normal text style
D_LT_STR        =$3758-$1A  ; display length-byte normal text string
M_CHARPUT       =$37E8-$1A  ; display %A on screen, menu style
D_ZM_STR        =$37EE-$1A  ; display zero-terminated string,  menu style
SCROLL_UP       =$37A0-$1A  ; scroll text screen up
TR_CHARPUT      =$3752-$1A  ; raw display of %A on screen, normal text style
CLEARLCD        =$38B4-$1A  ; clear LCD, but not text or graphics memory
D_HL_DECI       =$3896-$1A  ; disp. HL as 5-byte, right just., blank-pad. decimal
CLEARTEXT_F     =$37BE-$1A  ; clear LCD and text memory (affected by 1,(IY+13))
CLEARTEXT       =$37BE-$1A  ; clear LCD and text memory (affected by 1,(IY+13))
D_ZT_STR        =$3914-$1A  ; display zero-terminated normal text string
BUSY_OFF        =$3932-$1A  ; turn off "busy" indicataor
BUSY_ON         =$394A-$1A  ; turn on "busy" indicator

#ENDIF


;RAM locations
KEY_0          =$8000    ; translated scancode of last key, but 0 if gotten
KEY_1          =$8001    ; translated scancode of key down now
KEY_2          =$8002    ; same as 8001, but $FF if more than one key is down
KEY_STAT       =$8004    ; bit 2 set = key down now
LAST_KEY       =$8006    ; last key pressed
CONTRAST       =$8008    ; contrast (0..1Fh)
CURSOR_POS     =$800C    ; text position for word ops
CURSOR_ROW     =$800C    ; text cursor row
CURSOR_COL     =$800D    ; text cursor column
BUSY_COUNTER   =$8026    ; counter for busy indicator
BUSY_BITMAP    =$8027    ; bitmap for busy indicator
OP1            =$8028    ; OP1
OP2            =$8033    ; OP2
OP3            =$803E    ; OP3
OP4            =$8049    ; OP4
OP5            =$8054    ; OP5
OP6            =$805F    ; OP6
TEXT_MEM       =$808F    ; text memory
CURSOR_X       =$8215    ; x value of cursor
CURSOR_Y       =$8216    ; y value of cursor
_IY_TABLE      =$8528    ; where IY usually points
TEXT_MEM2      =$8BDF    ; secondary text memory
WIN_START      =$8C8F    ; number of first row in window
WIN_END        =$8C90    ; number of last row in window
GRAPH_MEM      =$88B8    ; Graph memory buffer - used in CALL_(_GRBUFCPY_V)
STACK          =$FFFF    ; Beginning of STACK (also works backwards)


; ROM routines
LD_HL_MHL       =$0033  ; HL=(HL), A=L
CP_HL_DE        =$0095  ; CP HL,DE (modifies flags only)
UNPACK_HL       =$00A1  ; unpacks one digit of HL into %A
STORE_KEY       =$01C7  ; store immediate keystroke from %A
GET_KEY         =$01D4  ; get immediate keystroke into %A
DISP_DELAY      =$07F3  ; delay for display controller

; FIND_PIXEL has been removed due to conflicts between shells
;FIND_PIXEL      =$4166  ; Find pixel (rom page most be 4 otherwise crash)
				    ; Located in the smae place for all the rom versions !

KEY_HAND        =$393E-$1A  ; get key using key.handler (using 2nd,aplha +contrast)
GET_T_CUR       =$37E2-$1A  ; HL = absolute address of text cursor
LAST_LINE       =$378E-$1A  ; Is it the last line ?
NEXT_LINE       =$3794-$1A  ; goto next line (might scroll)
CLEARTEXT_W     =$37CA-$1A  ; clear LCD and text memory,but only current window (affected by 1,(IY+13))
CLEAR_DISP      =$38E4-$1A  ; clear display
DONE            =$3884-$1A  ; Print DONE at end of current line
UP_TEXT         =$380C-$1A  ; Update text mem from command shadow

;TI-82 Hardware I/O Ports - USE AT YOUR OWN RISK
LINKPORT        =$00    ; Same functionality as the TI-85 port
KEYPAD          =$01    ; Same as TI-85
MISCPORT        =$02    ; ROM page switching, other functions...
LCDPOWER        =$03    ; Same as TI-85
MISCPORT2       =$04    ; Apparently same as TI-85 port
LCDINSTPORT     =$10    ; Apparently same as TI-83 port
LCDDATAPORT     =$11    ; Not much info on this one (TI-83?)

;Misc
#DEFINE TEXT_START RES 1,(IY+0Dh) \ RES 2,(IY+0Dh) \ ROM_CALL2(CLEAR_DISP) \ LD HL,0000h \ LD (CURSOR_POS),HL
#DEFINE TEXT_END SET 2,(IY+0Dh) \ SET 1,(IY+0Dh)
#DEFINE ROM_GETKEY  ROM_CALL2(KEY_HAND) \ RES 4,(IY+09)

;Codes returned by ROM_GETKEY
K_ON            =$00    ; NOTE:  You will have to RES 4,(IY+09) after pressed
R_ARROW         =$01
L_ARROW         =$02
U_ARROW         =$03
D_ARROW         =$04
K_ENTER         =$05
K_CLEAR         =$06
K_QUIT          =$40    ; [2nd] [MODE]/QUIT
